/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include <string.h>

#include "viper.h"
#include "viper_keyboard.h"
#include "viper_states.h"
/* #include "viper_gpm.h" */

void viper_window_set_key_func(WINDOW *window,VIPER_WKEY_FUNC func)
{
	extern VIPER   *viper;
	VIPER_WND	   *viper_wnd;
	
	if(window==NULL) return;
/*	viper=viper_get_instance(); */
	if(viper->wnd_count==0) return;

	viper_wnd=viper_get_viper_wnd(window);
	if(viper_wnd!=NULL) viper_wnd->key_func=func;
	
	return;
}

VIPER_WKEY_FUNC viper_window_get_key_func(WINDOW *window)
{
	extern VIPER		*viper;
	VIPER_WND			*viper_wnd;
	VIPER_WKEY_FUNC	func=NULL;
	
	if(window==NULL) return NULL;
		
/*	viper=viper_get_instance(); */
	if(viper->wnd_count==0) return NULL;

	viper_wnd=viper_get_viper_wnd(window);
	if(viper_wnd!=NULL) func=viper_wnd->key_func;
	
	return func;
}

gint32 viper_keystroke_pop(void)
{
	gint32	keystroke=0;
	gint32	key_code=0;

	key_code=getch();
	if(key_code==-1) return -1;
	
	if(key_code!=27) return key_code;
	
	keystroke=27;
	key_code=getch();
	if(key_code==-1) return keystroke;
	keystroke=keystroke | (key_code<<8);

	key_code=getch();
	if(key_code==-1) return keystroke;
	keystroke=keystroke | (key_code<<16);

	key_code=getch();
	if(key_code==-1) return keystroke;
	keystroke=keystroke | (key_code<<24);

	return keystroke;
}

gint32 viper_keystroke_get(void)
{
	gint32						keystroke;
	gint32						retval;
#ifdef _VIPER_MOUSE_GPM
  	static WINDOW				*event_window=NULL;
	static gint					event_mode=0;
	static MEVENT				new_mevent,old_mevent;
	gint							beg_x,beg_y;
	gint							max_x,max_y;
#endif

	keystroke=viper_keystroke_pop();
	retval=keystroke;
	
	if(keystroke==KEY_RESIZE)
	{
		viper_event_run(VIPER_EVENT_BROADCAST,"term-resized");
		viper_screen_redraw(REDRAW_ALL | REDRAW_BACKGROUND);
		keystroke=-1;
	}
	
#ifdef _VIPER_MOUSE_GPM		
	if(keystroke==KEY_MOUSE)
	{
		getmouse(&new_mevent);
					
		if((new_mevent.bstate & REPORT_MOUSE_POSITION)
			 && event_mode==EVENTMODE_MOVE)
		{
			viper_mvwin_rel(event_window,new_mevent.x-old_mevent.x,
				new_mevent.y-old_mevent.y);
			memcpy(&old_mevent,&new_mevent,sizeof(MEVENT));
		}
			
		if((new_mevent.bstate & REPORT_MOUSE_POSITION) 
			&& event_mode==EVENTMODE_RESIZE)
		{
			viper_wresize_rel(event_window,new_mevent.x-old_mevent.x,
				new_mevent.y-old_mevent.y);
			memcpy(&old_mevent,&new_mevent,sizeof(MEVENT));
		}
			
		/* check for a button press and a window hit */
		if((new_mevent.bstate & BUTTON1_PRESSED) && event_mode==EVENTMODE_IDLE)
		{
			event_window=viper_deck_hit_test(new_mevent.x,new_mevent.y);
			if(event_window!=NULL)
			{
				viper_window_set_top(event_window);
				memcpy(&old_mevent,&new_mevent,sizeof(MEVENT));
				getbegyx(WINDOW_FRAME(event_window),beg_y,beg_x);
				getmaxyx(WINDOW_FRAME(event_window),max_y,max_x);
				if(new_mevent.x==(beg_x+max_x-1) &&
					new_mevent.y==(beg_y+max_y)-1) event_mode=EVENTMODE_RESIZE;
				else event_mode=EVENTMODE_MOVE;
			}
			else event_mode=EVENTMODE_IDLE;
		}
					
		if(new_mevent.bstate & BUTTON1_RELEASED)
		{
			if(!(old_mevent.bstate & REPORT_MOUSE_POSITION))
			{
				if(event_mode==EVENTMODE_MOVE) viper_mvwin_rel(event_window,
					new_mevent.x-old_mevent.x,new_mevent.y-old_mevent.y);
				
            /* resize window  */		
				if(event_mode==EVENTMODE_RESIZE)
				{
					viper_wresize_rel(event_window,new_mevent.x-old_mevent.x,
						new_mevent.y-old_mevent.y);
					viper_screen_redraw(REDRAW_ALL);
				}
			}
			
			event_window=NULL;
			event_mode=EVENTMODE_IDLE;
		}
		
		if(new_mevent.bstate & BUTTON1_CLICKED)
		{
			event_window=viper_deck_hit_test(new_mevent.x,new_mevent.y);
			if(event_window!=NULL)
			{
				viper_window_set_top(event_window);
				viper_window_redraw(event_window);
			
				getbegyx(WINDOW_FRAME(event_window),beg_y,beg_x);
				getmaxyx(WINDOW_FRAME(event_window),max_y,max_x);
				if(new_mevent.x==(beg_x+max_x-2) && new_mevent.y==beg_y)
				{
					viper_window_close(event_window);
					keystroke=-1;
				}
				if(new_mevent.x==(beg_x+max_x-4) && new_mevent.y==beg_y)
				{
					viper_window_hide(event_window);
					viper_deck_cycle(VECTOR_BOTTOM_TO_TOP);
					keystroke=-1;
				}
			}
			event_window=NULL;
			event_mode=EVENTMODE_IDLE;
		}
		
		if(new_mevent.bstate & BUTTON1_DOUBLE_CLICKED)
		{
			event_window=viper_deck_hit_test(new_mevent.x,new_mevent.y);
			if(event_window!=NULL)
			{
				viper_window_set_top(event_window);
				viper_window_redraw(event_window);
			}
			
			event_window=NULL;
			event_mode=EVENTMODE_IDLE;
		}
	}
	
	if(keystroke==KEY_MOUSE)
	{
		if(new_mevent.bstate & BUTTON1_PRESSED) keystroke=-1;
		if(new_mevent.bstate & BUTTON1_RELEASED) keystroke=-1;
	}
#endif
	
	return retval;
}

gint viper_keystroke_dispatch(gint32 keystroke)
{
	static VIPER_WKEY_FUNC	func;

	if(keystroke==-1) return -1;
	
	func=viper_window_get_key_func(TOPMOST_WINDOW);
	if(func!=NULL)
	{
		func(keystroke,(gpointer)TOPMOST_WINDOW);
		return 1;
	}

	return 0;
}




