/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include "viper.h"
#include "viper_mouse.h"
#include "viper_screen.h"

#include <string.h>
/* #include <gpm.h> */

#include <poll.h>
/* #include <sys/time.h> */

#ifdef _VIPER_MOUSE_GPM
void viper_gpm_mouse_show(void)
{
	extern VIPER		*viper;
   extern WINDOW     *SCREEN_WINDOW;
	WINDOW				*screen_window;
	static Gpm_Event	old_event, new_event;
	MEVENT				m_event;
	chtype				color;
	gshort				fg,bg;
	static gint			viper_gpm_fd=-1;
	extern gint			gpm_flag;
	extern gint			gpm_tried;
	struct pollfd		fdpoll;
	gint					retval;
	
	/*	These simple checks correct a number of problems.
		First, we need to see if gpm has been started.  If not, it is
		imperative that we not try to override the connection.  If curses
		makes a connection after we have started then all chaos will break
		out.  Secondly, we need to check that we are not running under xterm
		(gpm_fd == -2).  If we try to persistently connect under xterm,
		there will be an barrage of error messages printed on stderr	*/
		
	if(gpm_fd==-2 || gpm_flag==0 || !gpm_tried) return;
	/* have previous attempts to connect failed? */
	if(gpm_fd==-1 && gpm_tried) return;
	
	/*	establish connection on first run.	*/
	if(viper_gpm_fd==-1) viper_gpm_fd=viper_gpm_connection_open();

/*	viper=viper_get_instance(); */
	screen_window=SCREEN_WINDOW;
	
	fdpoll.fd=viper_gpm_fd;
	fdpoll.events=POLLIN;
	retval=poll(&fdpoll,1,1);
	
	if(retval>0)
	{
 		Gpm_GetEvent(&new_event);
		viper_gpm_event_translate(&new_event,&m_event);
		ungetmouse(&m_event);
		new_event.x--;
		new_event.y--;
			
		if(new_event.x!=old_event.x || new_event.y!=old_event.y)
		{
			if(viper->console_mouse==NULL)
			{
				viper->console_mouse=newwin(1,1,0,0);
				color=mvwinch(screen_window,0,0);
				pair_content(PAIR_NUMBER(color & A_COLOR),&fg,&bg);
				if(bg==COLOR_RED || bg==COLOR_YELLOW || bg==COLOR_MAGENTA)
					color=VIPER_COLORS(COLOR_CYAN,COLOR_CYAN);
				if(bg==COLOR_CYAN)
					color=VIPER_COLORS(COLOR_YELLOW,COLOR_YELLOW);

				mvwaddch(viper->console_mouse,0,0,' ' | color);
			}
			
			color=mvwinch(screen_window,new_event.y,new_event.x);
			pair_content(PAIR_NUMBER(color & A_COLOR),&fg,&bg);
			if(bg==COLOR_RED || bg==COLOR_YELLOW || bg==COLOR_MAGENTA) 
				color=VIPER_COLORS(COLOR_CYAN,COLOR_CYAN);
			else
				color=VIPER_COLORS(COLOR_YELLOW,COLOR_YELLOW);
			mvwaddch(viper->console_mouse,0,0,' ' | color);
			
			mvwin(viper->console_mouse,new_event.y,new_event.x);
			viper_screen_redraw(REDRAW_ALL);
			memcpy(&old_event,&new_event,sizeof(Gpm_Event));
		}
	}

	return;
}

gint viper_gpm_connection_open(void)
{
	Gpm_Connect		gpm_connect;
	gint			viper_gpm_fd;
	
	memset(&gpm_connect,0,sizeof(gpm_connect));
	gpm_connect.eventMask=GPM_MOVE | GPM_UP | GPM_DOWN | GPM_DRAG;
	gpm_connect.maxMod=~0;
	viper_gpm_fd=Gpm_Open(&gpm_connect,0);

   /* if(viper_gpm_fd==-1) flash(); */
	
	return viper_gpm_fd;
}

void viper_gpm_event_translate(Gpm_Event *gpm_event,MEVENT *mevent)
{
	mevent->id=0;
	mevent->bstate=0;
	
	mevent->bstate=gpm_event->modifiers;
	mevent->x=gpm_event->x-1;
	mevent->y=gpm_event->y-1;
	
	/* if(GPM_BARE_EVENTS(gpm_event->type)) */
	if((gpm_event->type & 0x0f)==GPM_DOWN)
	{
		if(gpm_event->buttons & GPM_B_LEFT)
			mevent->bstate |= BUTTON1_PRESSED;
		if(gpm_event->buttons & GPM_B_MIDDLE)
			mevent->bstate |= BUTTON2_PRESSED;
		if(gpm_event->buttons & GPM_B_RIGHT)
			mevent->bstate |= BUTTON3_PRESSED;
	}
	
	if((gpm_event->type & 0x0f)==GPM_UP)
	{
		if(gpm_event->buttons & GPM_B_LEFT)
			mevent->bstate |= BUTTON1_RELEASED;
		if(gpm_event->buttons & GPM_B_MIDDLE)
			mevent->bstate |= BUTTON2_RELEASED;
		if(gpm_event->buttons & GPM_B_RIGHT)
			mevent->bstate |= BUTTON3_RELEASED;
	}
	
	/*	not sure about the validity of using REPORT_MOUSE_POSITION in this
		way but it seems to not cause problems */
	if((gpm_event->type & 0x0f)==GPM_DRAG)
	{
		mevent->bstate |= REPORT_MOUSE_POSITION;
	}
	
	if(Gpm_StrictDouble(gpm_event->type) && (gpm_event->type & GPM_UP))
	{
		if(gpm_event->buttons & GPM_B_LEFT)
			mevent->bstate = BUTTON1_DOUBLE_CLICKED;
		if(gpm_event->buttons & GPM_B_MIDDLE)
			mevent->bstate = BUTTON1_DOUBLE_CLICKED;
		if(gpm_event->buttons & GPM_B_RIGHT)
			mevent->bstate = BUTTON1_DOUBLE_CLICKED;
		return;
	}
	
	if(Gpm_StrictSingle(gpm_event->type) && (gpm_event->type & GPM_UP))
	{
		if(gpm_event->buttons & GPM_B_LEFT)
			mevent->bstate = BUTTON1_CLICKED;
		if(gpm_event->buttons & GPM_B_MIDDLE)
			mevent->bstate = BUTTON1_CLICKED;
		if(gpm_event->buttons & GPM_B_RIGHT)
			mevent->bstate = BUTTON1_CLICKED;
		return;
	}
		
	return;
}
#endif
